@@ -22,7 +22,6 @@ from docopt import docopt
 from email.mime.base import MIMEBase
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
-from email import generator
 from hashlib import md5
 from imio.email.dms.imap import IMAPEmailHandler
 from imio.email.dms.utils import safe_unicode
@@ -39,6 +38,7 @@ import six
 import sys
 import tarfile
 import zc.lockfile
+from imio.email.dms.utils import save_as_eml
 
 try:
     from pathlib import Path
@@ -81,6 +81,30 @@ If you are using Mozilla Thunderbird:\n
 Please excuse us for the inconvenience.\n
 """
 
+UNSUPPORTED_ORIGIN_EMAIL = u"""
+Cher utilisateur d'iA.Docs,
+
+Le transfert de l'email attaché a été rejeté car il n'a pas été transféré correctement.\n
+Veuillez refaire le transfert du mail original en transférant "en tant que pièce jointe".\n
+Si vous utilisez Microsoft Outlook:\n
+- Dans le ruban, cliquez sur la flèche du ménu déroulant située sur le bouton de transfert\n
+- Choisissez le transfert en tant que pièce jointe\n
+- Envoyez le mail sans rien compléter d'autre à l'adresse prévue pour iA.Docs.\n
+\n
+Si vous utilisez Mozilla Thunderbird:\n
+- Faites un clic droit sur l'email pour ouvrir le menu contextuel\n
+- Sélectionnez "Transférer au format" > "Pièce jointe".\n
+- Envoyez le mail sans rien compléter d'autre à l'adresse prévue pour iA.Docs.\n
+\n
+Cordialement.\n
+"""
+
+RESULT_MAIL = u"""
+Client ID : {0}
+IMAP login : {1}\n
+{2}\n
+"""
+
 
 class DmsMetadataError(Exception):
     """ The response from the webservice dms_metadata route is not successful """
@@ -130,6 +154,7 @@ def notify_unsupported_origin(config, mail, from_):
 
     msg = MIMEMultipart()
     msg["Subject"] = "Error importing email into iA.docs"
+    msg["Subject"] = "Erreur de transfert de votre email dans iA.Docs"
     msg["From"] = sender
     msg["To"] = from_
 
@@ -147,6 +172,27 @@ def notify_unsupported_origin(config, mail, from_):
     smtp.quit()
 
 
+def notify_result(config, subject, message):
+    client_id = config["webservice"]["client_id"]
+    login = config["mailbox"]["login"]
+    smtp_infos = config["smtp"]
+    sender = smtp_infos["sender"]
+    recipient = smtp_infos["recipient"]
+
+    msg = MIMEMultipart()
+    msg["Subject"] = "{} for client {}".format(subject, client_id)
+    msg["From"] = sender
+    msg["To"] = recipient
+
+    main_text = MIMEText(RESULT_MAIL.format(client_id, login, message), "plain")
+    msg.attach(main_text)
+
+    smtp = SMTP(str(smtp_infos["host"]), int(smtp_infos["port"]))
+    msg_content = msg.as_string().encode("utf8") if six.PY3 else msg.as_string()
+    smtp.sendmail(sender, recipient, msg_content)
+    smtp.quit()
+
+
 def get_mailbox_infos(config):
     mailbox_infos = config["mailbox"]
     host = str(mailbox_infos["host"])
@@ -164,7 +210,7 @@ def get_preview_pdf_path(config, mail_id):
     return os.path.join(output_dir, filename)
 
 
-def send_to_ws(config, headers, pdf_path, attachments, mail_id):
+def send_to_ws(config, headers, main_file_path, attachments, mail_id):
     ws = config["webservice"]
     client_id = "{0}Z{1}".format(ws['client_id'][:2], ws['client_id'][-4:])
     counter_dir = Path(ws['counter_dir'])
@@ -173,15 +219,16 @@ def send_to_ws(config, headers, pdf_path, attachments, mail_id):
         next_id = int(next_id_path.read_text()) + 1
     else:
         next_id = 1
-
     external_id = "{0}{1:08d}".format(client_id, next_id)
+
     tar_path = Path('/tmp') / '{}.tar'.format(external_id)
     with tarfile.open(str(tar_path), "w") as tar:
-        # 1) email pdf printout
-        pdf_contents = Path(pdf_path).open('rb').read()
-        pdf_info = tarfile.TarInfo(name='email.pdf')
-        pdf_info.size = len(pdf_contents)
-        tar.addfile(tarinfo=pdf_info, fileobj=BytesIO(pdf_contents))
+        # 1) email pdf printout or eml file
+        mf_contents = Path(main_file_path).open('rb').read()
+        basename, ext = os.path.splitext(main_file_path)
+        mf_info = tarfile.TarInfo(name='email{}'.format(ext))
+        mf_info.size = len(mf_contents)
+        tar.addfile(tarinfo=mf_info, fileobj=BytesIO(mf_contents))
 
         # 2) metadata.json
         metadata_contents = json.dumps(headers).encode("utf8") if six.PY3 else json.dumps(headers)
@@ -279,13 +326,11 @@ def process_mails():
         logger.info(parsed.headers)
         message = parsed.message
         filename = '{}.eml'.format(mail_id)
-        if '--get_eml_orig' in arguments:
+        if arguments.get('--get_eml_orig'):
             message = parsed.initial_message
             filename = '{}_o.eml'.format(mail_id)
         logger.info('Writing {} file'.format(filename))
-        with open(filename, 'w') as emlfile:
-            gen = generator.Generator(emlfile)
-            gen.flatten(message)
+        save_as_eml(filename, message)
         handler.disconnect()
         lock.close()
         sys.exit()
@@ -307,7 +352,7 @@ def process_mails():
     for mail_info in handler.get_waiting_emails():
         mail_id = mail_info.id
         mail = mail_info.mail
-        pdf_path = get_preview_pdf_path(config, mail_id)
+        main_file_path = get_preview_pdf_path(config, mail_id)
         try:
             parser = Parser(mail)
             if parser.origin == 'Generic inbox':
@@ -318,8 +363,13 @@ def process_mails():
                 continue
             headers = parser.headers
             attachments = parser.attachments
-            parser.generate_pdf(pdf_path)
-            send_to_ws(config, headers, pdf_path, attachments, mail_id)
+            try:
+                parser.generate_pdf(main_file_path)
+            except Exception as pdf_exc:
+                # if 'XDG_SESSION_TYPE=wayland' not in str(pdf_exc):
+                main_file_path = main_file_path.replace('.pdf', '.eml')
+                save_as_eml(main_file_path, parser.message)
+            send_to_ws(config, headers, main_file_path, attachments, mail_id)
             handler.mark_mail_as_imported(mail_id)
             imported += 1
         except Exception as e:
@@ -368,7 +418,9 @@ def clean_mails():
         sys.exit()
     deleted = ignored = error = 0
     mail_ids = data[0].split()
-    logger.info("Get '{}' emails older than '{}'".format(len(mail_ids), before_date))
+    mail_ids_len = len(mail_ids)
+    out = [u"Get '{}' emails older than '{}'".format(mail_ids_len, before_date)]
+    logger.info("Get '{}' emails older than '{}'".format(mail_ids_len, before_date))
     # sys.exit()
     for mail_id in mail_ids:
         res, flags_data = handler.connection.fetch(mail_id, '(FLAGS)')
@@ -386,15 +438,20 @@ def clean_mails():
             continue
         parser = Parser(mail)
         logger.info(u"{}: '{}'".format(mail_id, parser.headers['Subject']))
+        out.append(u"{}: '{}'".format(mail_id, parser.headers['Subject']))
         if doit:
             handler.connection.store(mail_id, "+FLAGS", "\\Deleted")
         deleted += 1
     if deleted:
-        logger.info("Get '{}' emails older than '{}'".format(len(mail_ids), before_date))
+        logger.info("Get '{}' emails older than '{}'".format(mail_ids_len, before_date))
         if doit:
             res, data = handler.connection.expunge()
             if res != "OK":
+                out.append(u"ERROR: Unable to deleted mails !!")
                 logger.error("Unable to deleted mails")
     handler.disconnect()
+    out.append(u"{} emails have been deleted. {} emails are ignored. {} emails have caused an error.".format(
+               deleted, ignored, error))
     logger.info("{} emails have been deleted. {} emails are ignored. {} emails have caused an error.".format(
                 deleted, ignored, error))
+    notify_result(config, 'Result of clean_mails', u'\n'.join(out))
